# Preprocessing

##################################################
# Short reads

# Preprocess the short reads using fastp
rule fastp_sr:
    input:
        r1=os.path.join(RESULTS_DIR, "input/{mtype}/sr/R1.fq.gz"),
        r2=os.path.join(RESULTS_DIR, "input/{mtype}/sr/R2.fq.gz")
    output:
        r1=os.path.join(RESULTS_DIR, "preproc/{mtype}/sr/R1.fastp.fastq.gz"),
        r2=os.path.join(RESULTS_DIR, "preproc/{mtype}/sr/R2.fastp.fastq.gz"),
        html=os.path.join(RESULTS_DIR, "preproc/{mtype}/sr/fastp.html"),
        json=os.path.join(RESULTS_DIR, "preproc/{mtype}/sr/fastp.json")
    log:
        out="logs/fastp.{mtype}.sr.out.log",
        err="logs/fastp.{mtype}.sr.err.log"
    wildcard_constraints:
        mtype="metag|metat"
    threads:
        config["fastp"]["threads"]
    conda:
        os.path.join(ENV_DIR, "fastp.yaml")
    message:
        "Preprocessing short reads: FastP"
    shell:
        "(date && fastp -l {config[fastp][min_length]} -i {input.r1} -I {input.r2} -o {output.r1} -O {output.r2} -h {output.html} -j {output.json} -w {threads} && date) 2> {log.err} > {log.out}"

rule fastqc_fastp_sr:
    input:
        os.path.join(RESULTS_DIR, "preproc/{mtype}/sr/{rid}.fastp.fastq.gz")
    output:
        html=os.path.join(RESULTS_DIR, "qc/{mtype}/sr/{rid}.fastp_fastqc.html"),
        zip=os.path.join(RESULTS_DIR, "qc/{mtype}/sr/{rid}.fastp_fastqc.zip")
    log:
        out="logs/fastqc.{mtype}.sr.{rid}.out.log",
        err="logs/fastqc.{mtype}.sr.{rid}.err.log"
    wildcard_constraints:
        mtype="metag|metat",
        rid="|".join(["R1", "R2"])
    threads:
        config["fastqc"]["threads"]
    conda:
        os.path.join(ENV_DIR, "fastqc.yaml")
    message:
        "Preprocessing short reads: FastQC"
    shell:
        "(date && fastqc {config[fastqc][params]} -t {threads} -o $(dirname {output.html}) {input} && date) 2> {log.err} > {log.out}"

##################################################
# Long reads

# Basecalling
# NOTE: in basecalling.smk

# QC
rule nanostat_guppy_basecalling:
    input:
        os.path.join(RESULTS_DIR, "preproc/metag/lr/lr.fastq.gz")
    output:
        os.path.join(RESULTS_DIR, "qc/metag/lr/NanoStats.txt")
    log:
        out="logs/nanostats.metag.lr.out.log",
        err="logs/nanostats.metag.lr.err.log"
    conda:
        os.path.join(ENV_DIR, "nanostat.yaml")
    message:
        "Preprocessing long reads: NanoStats"
    shell:
        "(date && NanoStat --fastq {input} --outdir $(dirname {output}) -n $(basename {output}) && date) 2> {log.err} > {log.out}"
