# CASC
rule install_casc:
    output:
        os.path.join(MOD_DIR, "casc/bin/casc")
    log:
        out="logs/casc_install.out.log",
        err="logs/casc_install.err.log"
    params:
        path=os.path.join(MOD_DIR, "casc")
    conda:
        os.path.join(ENV_DIR, "casc.yaml")
    message:
        "Setup: install CASC"
    shell:
        "(cd {params.path} && "
        "perl Makefile.PL PREFIX=\"$(realpath .)\" && "
        "make && "
        "make test && "
        "make install) 2> {log.err} > {log.out}"

# MinCED
rule install_minced:
    output:
        os.path.join(MOD_DIR, "minced/minced.jar")
    log:
        out="logs/minced_install.out.log",
        err="logs/minced_install.err.log"
    params:
        path=os.path.join(MOD_DIR, "minced")
    conda:
        os.path.join(ENV_DIR, "minced.yaml")
    message:
        "Setup: install MinCED"
    shell:
        "(cd {params.path} && "
        "make && "
        "make test) 2> {log.err} > {log.out}"

# Download RGI data
rule download_rgi_db:
    output:
        archive=temp(os.path.join(DB_DIR, "rgi/card-data.tar.bz2")),
        json=os.path.join(DB_DIR, "rgi/card.json")
    log:
        out="logs/rgi_db.out.log",
        err="logs/rgi_db.err.log"
    params:
        db_url=config["rgi"]["db_url"]
    message:
        "Setup: download RGI data"
    shell:
        "(date && "
        "wget -O {output.archive} {params.db_url} --no-check-certificate && "
        "tar -C $(dirname {output.archive}) -xvf {output.archive} && "
        "date) 2> {log.err} > {log.out}"

# Setup RGI: load required DB
# NOTE: to make sure that the same DB is used for all targets
rule setup_rgi_db:
    input:
        os.path.join(DB_DIR, "rgi/card.json")
    output:
        "status/rgi_setup.done"
    log:
        out="logs/rgi_setup.out.log",
        err="logs/rgi_setup.err.log"
    conda:
        os.path.join(ENV_DIR, "rgi.yaml")
    message:
        "Setup: load RGI DB"
    shell:
        "(rgi clean --local && "
        "rgi load --card_json {input} --local && "
        "rgi database --version --local) 2> {log.err} > {log.out} && "
        "touch {output}"

# MUMMER
# rule install_mummer:
#     output:
#         src=temp()
#         bin=???
#     log:
#         out="logs/mummer_install.out.log",
#         err="logs/mummer_install.err.log"
#     params:
#         archive=config["mummer"]["archive"]
#     conda:
#         os.path.join(ENV_DIR, "mummer.yaml")
#     message:
#         "Setup: install MUMMER"
#     shell:
#         "(wget {config[mummer][source]} -O $(dirname $(dirname {output}))/$(basename {params.archive}) &&"
#         "cd $(dirname $(dirname {output})) && "
#         "tar -xzvf $(basename {params.archive}) && "
#         "cd $(basename -s .tar.gz {params.archive}) && "
#         "./configure --prefix=$(dirname {output}) && "
#         "make && "
#         "make install) 2> {log.err} > {log.out}"