# Taxonomic analysis

rule kraken2_contigs:
    input:
        contigs=os.path.join(RESULTS_DIR, "assembly/{rtype}/{tool}/ASSEMBLY.fasta"),
        db=lambda wildcards: config["kraken2"]["db"][wildcards.db]
    output:
        labels=os.path.join(RESULTS_DIR, "taxonomy/kraken2/{rtype}_{tool}_{db}_labels.txt"),
        report=os.path.join(RESULTS_DIR, "taxonomy/kraken2/{rtype}_{tool}_{db}_report.txt")
    log:
        out="logs/kraken2.{db}.{rtype}.{tool}.out.log",
        err="logs/kraken2.{db}.{rtype}.{tool}.err.log"
    wildcard_constraints:
        rtype="|".join(READ_TYPES),
        tool="|".join(ASSEMBLERS),
        db="|".join(config["kraken2"]["db"].keys())
    threads:
        config["kraken2"]["threads"]
    params:
        params=config["kraken2"]["class"]["contigs"]
    conda:
        "../envs/kraken2.yaml"
    message:
        "Classification w/ Kraken2 ({wildcards.db})"
    shell:
        "(date && "
        "kraken2 --db {input.db} --threads {threads} --use-names {params.params} --report {output.report} --output {output.labels} {input.contigs} && "
        "date) 2> {log.err} > {log.out}"

rule kraken2_sr:
    input:
        r1=os.path.join(RESULTS_DIR, "preproc/{mtype}/sr/R1.fastp.fastq.gz"),
        r2=os.path.join(RESULTS_DIR, "preproc/{mtype}/sr/R2.fastp.fastq.gz"),
        db=lambda wildcards: config["kraken2"]["db"][wildcards.db]
    output:
        labels=os.path.join(RESULTS_DIR, "taxonomy/kraken2/{mtype}_sr_{db}_labels.txt"),
        report=os.path.join(RESULTS_DIR, "taxonomy/kraken2/{mtype}_sr_{db}_report.txt")
    log:
        out="logs/kraken2.{db}.{mtype}.sr.out.log",
        err="logs/kraken2.{db}.{mtype}.sr.err.log"
    wildcard_constraints:
        mtype="|".join(META_TYPES),
        db="|".join(config["kraken2"]["db"].keys())
    threads:
        config["kraken2"]["threads"]
    params:
        params=config["kraken2"]["class"]["sr"]
    conda:
        "../envs/kraken2.yaml"
    message:
        "Classification w/ Kraken2 ({wildcards.db})"
    shell:
        "(date && "
        "kraken2 --db {input.db} --threads {threads} --use-names {params.params} --report {output.report} --output {output.labels} {input.r1} {input.r2} && "
        "date) 2> {log.err} > {log.out}"

rule kraken2_lr:
    input:
        lr=os.path.join(RESULTS_DIR, "preproc/metag/lr/lr.fastq.gz"),
        db=lambda wildcards: config["kraken2"]["db"][wildcards.db]
    output:
        labels=os.path.join(RESULTS_DIR, "taxonomy/kraken2/metag_lr_{db}_labels.txt"),
        report=os.path.join(RESULTS_DIR, "taxonomy/kraken2/metag_lr_{db}_report.txt")
    log:
        out="logs/kraken2.{db}.metag.lr.out.log",
        err="logs/kraken2.{db}.metag.lr.err.log"
    wildcard_constraints:
        rtype="|".join(READ_TYPES),
        tool="|".join(ASSEMBLERS),
        db="|".join(config["kraken2"]["db"].keys())
    threads:
        config["kraken2"]["threads"]
    params:
        params=config["kraken2"]["class"]["contigs"]
    conda:
        "../envs/kraken2.yaml"
    message:
        "Classification w/ Kraken2 ({wildcards.db})"
    shell:
        "(date && "
        "kraken2 --db {input.db} --threads {threads} --use-names {params.params} --report {output.report} --output {output.labels} {input.lr} && "
        "date) 2> {log.err} > {log.out}"