# Assembly

include:
    "../rules/analysis.smk"

# NOTE: Using "shell: touch ..." to avoid the rule from being autodetected as `localrule`.
#       This is needed so that an email can be sent upon event changes for this rule.

rule ANALYSIS:
    input:
        "status/analysis_assembly.done",
        "status/analysis_proteins.done",
        "status/analysis_circ.done",
        "status/analysis_mash.done"
    output:
        touch("status/analysis.done")

rule ANALYSIS_ASSEMBLY:
    input:
        expand(
            os.path.join(RESULTS_DIR, "analysis/quast/{rtype_tool}/report.tsv"),
            rtype_tool=["%s/%s" % (rtype, tool) for rtype, tool in READ_ASSEMBLERS]
        )
    output:
        touch("status/analysis_assembly.done")

rule ANALYSIS_PROTEINS:
    input:
        expand(
            os.path.join(RESULTS_DIR, "analysis/cdhit/{combi}.faa"),
            combi=["%s_%s__%s_%s" % (p[0][0], p[0][1], p[1][0], p[1][1]) for p in READ_ASSEMBLER_PAIRS]
        ),
        expand(
            os.path.join(RESULTS_DIR, "analysis/cdhit/{combi}.faa"),
            combi=["%s_%s__%s_%s" % (p[1][0], p[1][1], p[0][0], p[0][1]) for p in READ_ASSEMBLER_PAIRS]
        )
    output:
        touch("status/analysis_proteins.done")

rule ANALYSIS_CIRC:
    input:
        expand(
            os.path.join(RESULTS_DIR, "analysis/circ_blast/{rtype_tool}/ASSEMBLY.{ctype}.tsv"),
            rtype_tool=["%s/%s" % (rtype, tool) for rtype, tool in READ_ASSEMBLERS],
            ctype=["circ", "compl"]
        )
    output:
        touch("status/analysis_circ.done")

rule ANALYSIS_MASH:
    input:
        os.path.join(RESULTS_DIR, "analysis/mash/contigs.dist"),
        expand(
            os.path.join(RESULTS_DIR, "analysis/mash/screen.{rtype_tool}.tsv"),
            rtype_tool=["%s.%s" % (rtype, tool) for rtype, tool in READ_ASSEMBLERS]
        )
    output:
        touch("status/analysis_mash.done")
