#!/usr/bin/Rscript

# NOTE: code to be executed in report_render.R to generate data for report.Rmd

## DATA

##############################
# NanoStats
TABS$nanostats <- read_nanostats(snakemake@input$nanostats)

##############################
# Mappability

TABS$mappability <- list()
TABS$mappability$metag <- read_mappability(snakemake@input$mappability[1])
if(length(snakemake@input$mappability) > 1){
    TABS$mappability$metat <- read_mappability(snakemake@input$mappability[2])
} else {
    TABS$mappability$metat <- NULL
}

mappability_theme <-
    theme_bw() +
    theme(
        # grid
        panel.grid=element_blank(),
        # strip
        strip.background=element_rect(fill="white"),
        strip.text=element_text(size=12, color="black"),
        # axes
        axis.title=element_text(size=12, color="black"),
        axis.text.y=element_text(size=9, color="black"),
        axis.text.x=element_text(size=9, color="black", angle=90, vjust=0.5, hjust=1)
    )

FIGS$mappability <- list()
FIGS$mappability$metag <-
    ggplot(data=TABS$mappability$metag[TABS$mappability$metag$variable=="mapped_pct",], aes(x=tool, y=value, fill=tool)) +
    geom_col() +
    geom_text(aes(label=value, y=0.5*value), color="white") +
    scale_fill_manual(values=ASM_TOOL_COLORS, guide=NULL) +
    facet_wrap(vars(minlength), ncol=2, scales="fixed") +
    labs(
        x="",
        y="Mapped reads [%]"
    ) +
    mappability_theme

if(!is.null(TABS$mappability$metat)){
    FIGS$mappability$metat <-
        ggplot(data=TABS$mappability$metat[TABS$mappability$metat$variable=="mapped_pct",], aes(x=tool, y=value, fill=tool)) +
        geom_col() +
        geom_text(aes(label=value, y=0.5*value), color="white") +
        scale_fill_manual(values=ASM_TOOL_COLORS, guide=NULL) +
        facet_wrap(vars(minlength), ncol=2, scales="fixed") +
        labs(
            x="",
            y="Mapped reads [%]"
        ) +
        mappability_theme
}

##############################
# Mash

TABS$mash <- read_mash_dist(snakemake@input$mash)
mash_htree <- hclust(d=as.dist(TABS$mash), method="complete")
FIGS$mash <- pheatmap(
    TABS$mash,
    color=viridis(20),
    border_color="white",
    scale="none",
    cluster_rows=mash_htree,
    cluster_cols=mash_htree,
    silent=TRUE
)

##############################
# QAUST: assembly quality

TABS$quast <- read_quast(snakemake@input$quast)
TABS$quast_melted <- reshape2::melt(
    cbind(stat_vars=rownames(TABS$quast), TABS$quast),
    id.vars="stat_vars"
)
TABS$quast_melted$variable <- factor(TABS$quast_melted$variable, ordered=TRUE, levels=ASM_TOOL_NAMES)

FIGS$quast <-
    ggplot(data=TABS$quast_melted, aes(x=variable, y=value)) +
    geom_col(aes(fill=variable)) +
    scale_fill_manual(values=ASM_TOOL_COLORS, guide=NULL) +
    facet_wrap(vars(stat_vars), ncol=2, scales="free_y") +
    labs(
        x="",
        y="QUAST statistic"
    ) +
    theme_bw() +
    theme(
        # grid
        panel.grid=element_blank(),
        # strip
        strip.background=element_rect(fill="white"),
        strip.text=element_text(size=12, color="black"),
        # axes
        axis.title=element_text(size=12, color="black"),
        axis.text.y=element_text(size=9, color="black"),
        axis.text.x=element_text(size=9, color="black", angle=90, vjust=0.5, hjust=1)
    )

##############################
# Plasflow: plasmid prediction

TABS$plasflow <- read_plasflow(snakemake@input$plasflow)
TABS$plasflow_melted <- reshape2::melt(TABS$plasflow, id.vars=c("label", "type", "tool"), variable.name="statstype")
FIGS$plasflow <- list()

for(stype in c("sum_pct")){
    FIGS$plasflow[[stype]] <-
        ggplot(
            data=TABS$plasflow_melted[TABS$plasflow_melted$statstype == stype,],
            aes_string(x="tool", y="value", fill="label")
        ) +
        geom_col(position="dodge") +
        scale_fill_manual(values=PLASFLOW_COLORS$labels, guide="legend") +
        labs(
            x="",
            y=PLASFLOW_NAMES$statstype[stype]
        ) +
        theme_bw() +
        theme(
            # legend
            legend.title=element_blank(),
            # grid
            panel.grid=element_blank(),
            # axes
            axis.title=element_text(size=12, color="black"),
            axis.text.y=element_text(size=9, color="black"),
            axis.text.x=element_text(size=9, color="black", angle=90, vjust=0.5, hjust=1)
        )
}

##############################
# Prodigal

TABS$prodigal <- read_prodigal(snakemake@input$prodigal)
TABS$prodigal_melted <- reshape2::melt(TABS$prodigal[,c("tool", "total", "partial")], id.vars=c("tool"))

FIGS$prodigal <- list()
FIGS$prodigal$counts <-
    ggplot(data=TABS$prodigal_melted, aes(x=variable, y=value, fill=tool)) +
    geom_col(position="dodge") +
    scale_fill_manual(values=ASM_TOOL_COLORS) +
    labs(
        x="",
        y="Gene count"
    ) +
    theme_bw() +
    theme(
        # legend
        legend.title=element_blank(),
        # grid
        panel.grid=element_blank(),
        # axes
        axis.title=element_text(size=12, color="black"),
        axis.text.y=element_text(size=9, color="black"),
        axis.text.x=element_text(size=9, color="black", angle=90, vjust=0.5, hjust=1)
    )

FIGS$prodigal$pct <-
    ggplot(data=TABS$prodigal, aes(x=tool, y=partial_pct, fill=tool)) +
    geom_col() +
    scale_fill_manual(values=ASM_TOOL_COLORS) +
    labs(
        x="",
        y="Percentage of partial genes"
    ) +
    theme_bw() +
    theme(
        # legend
        legend.title=element_blank(),
        # grid
        panel.grid=element_blank(),
        # axes
        axis.title=element_text(size=12, color="black"),
        axis.text.y=element_text(size=9, color="black"),
        axis.text.x=element_text(size=9, color="black", angle=90, vjust=0.5, hjust=1)
    )

##############################
# DIAMOND

TABS$diamond <- read_diamond(snakemake@input$diamond)

FIGS$diamond <- list()
FIGS$diamond$normal <-
    ggplot(data=TABS$diamond, aes(x=tool, y=qs_len_ratio, fill=tool)) +
    geom_violin(draw_quantiles = c(0.25, 0.5, 0.75)) +
    scale_fill_manual(values=ASM_TOOL_COLORS, guide=NULL) +
    labs(
        x="",
        y="Query/subject length ratio"
    ) +
    theme_bw() +
    theme(
        # legend
        legend.title=element_blank(),
        # grid
        panel.grid=element_blank(),
        # axes
        axis.title=element_text(size=12, color="black"),
        axis.text.y=element_text(size=9, color="black"),
        axis.text.x=element_text(size=9, color="black", angle=90, vjust=0.5, hjust=1)
    )
FIGS$diamond$log <- FIGS$diamond$normal + scale_y_continuous(trans="log10")
FIGS$diamond$normal_zoom <- FIGS$diamond$normal + coord_cartesian(ylim=c(0, 1.5))
FIGS$diamond$density <-
    ggplot(data=TABS$diamond, aes(x=qs_len_ratio, colour=tool, fill=tool)) +
    geom_density(alpha=0.1) +
    scale_colour_manual(values=ASM_TOOL_COLORS, guide=NULL) +
    scale_fill_manual(values=ASM_TOOL_COLORS, guide=NULL) +
    facet_wrap(vars(tool), ncol=1, scales="free_y") +
    labs(
        x="Query/subject length ratio",
        y="Density"
    ) +
    theme_bw() +
    theme(
        # legend
        legend.title=element_blank(),
        # grid
        panel.grid=element_blank(),
        # strip
        strip.background=element_rect(fill="white"),
        strip.text=element_text(size=9, color="black"),
        # axes
        axis.title=element_text(size=12, color="black"),
        axis.text.y=element_text(size=9, color="black"),
        axis.text.x=element_text(size=9, color="black", angle=90, vjust=0.5, hjust=1)
    )

##############################
# CD-HIT

TABS$cdhit <- read_cdhit(snakemake@input$cdhit)

FIGS$cdhit <-
    ggplot(data=TABS$cdhit, aes(x=tool2, y=unique, fill=tool2)) +
    geom_col() +
    facet_wrap(vars(tool1), ncol=1) +
    scale_fill_manual(values=ASM_TOOL_COLORS, guide=NULL) +
    labs(
        x="Assembly 2",
        y="Proteins from Assembly 2 not in Assembly 1"
    ) +
    theme_bw() +
    theme(
        # grid
        panel.grid=element_blank(),
        # strip
        strip.background=element_rect(fill="white"),
        strip.text=element_text(size=9, color="black"),
        # axes
        axis.title=element_text(size=12, color="black"),
        axis.text.y=element_text(size=9, color="black"),
        axis.text.x=element_text(size=9, color="black", angle=90, vjust=0.5, hjust=1)
    )

##############################
# RGI: AMR prediction

TABS$rgi        <- read_rgi(snakemake@input$rgi)
TABS$rgi_melted <- reshape2::melt(TABS$rgi, id.vars=c("label", "col", "type"), variable.name="tool", value.name="count")
TABS$rgi_aggr   <- TABS$rgi_melted[TABS$rgi_melted$col == "ARO",]
TABS$rgi_aggr   <- aggregate(TABS$rgi_aggr$count, by=list(tool=TABS$rgi_aggr$tool, type=TABS$rgi_aggr$type), FUN=sum)
TABS$rgi_aggr$tool <- factor(TABS$rgi_aggr$tool, levels=ASM_TOOL_NAMES, ordered=TRUE)

FIGS$rgi  <- list()
rgi_theme <- 
    theme_bw() +
    theme(
        # grid
        panel.grid=element_blank(),
        # strip
        strip.background=element_rect(fill="white"),
        strip.text=element_text(size=9, color="black"),
        # axes
        axis.title=element_text(size=12, color="black"),
        axis.text.y=element_text(size=9, color="black"),
        axis.text.x=element_text(size=9, color="black", angle=90, vjust=0.5, hjust=1)
    )
for(ctype in unique(TABS$rgi_melted$type)){
    # Counts per feature type
    for(col in c("Resistance mechanism")){
        df <- TABS$rgi_melted[TABS$rgi_melted$type == ctype & TABS$rgi_melted$col == col,]
        FIGS$rgi[[sprintf("%s_%s", ctype, col)]] <-
            ggplot(data=df, aes(x=tool, y=count, fill=tool)) +
            geom_col() +
            scale_fill_manual(values=ASM_TOOL_COLORS, guide=NULL) +
            facet_wrap(vars(label), ncol=round(sqrt(length(unique(df$label)))), scales="free_y") +
            labs(
                x="",
                y=sprintf("%s (%s hits)", col, ctype)
            ) +
            rgi_theme
    }
}
# Total counts
FIGS$rgi[[sprintf("total")]] <-
    ggplot(data=TABS$rgi_aggr, aes(x=tool, y=x, fill=tool)) +
    geom_col() +
    scale_fill_manual(values=ASM_TOOL_COLORS, guide=NULL) +
    facet_wrap(vars(type), ncol=3) +
    labs(
        x="",
        y="Total hits"
    ) +
    rgi_theme

##############################
# CRISPR

TABS$casc   <- read_crispr(snakemake@input$casc)
TABS$minced <- read_crispr(snakemake@input$minced)
TABS$crispr <- rbind(TABS$casc, TABS$minced)
# aggregate: spacers per assembly
TABS$crispr_aggr_sa <- aggregate(
    x=TABS$crispr$spacers,
    by=list(crispr_tool=TABS$crispr$crispr_tool, asm_tool=TABS$crispr$asm_tool),
    FUN=sum
)
# aggregate: arrays per assembly
TABS$crispr_aggr_aa <- aggregate(
    x=TABS$crispr$seq_id,
    by=list(crispr_tool=TABS$crispr$crispr_tool, asm_tool=TABS$crispr$asm_tool),
    FUN=length
)

FIGS$crispr <- list()
crispr_theme <-
    theme_bw() +
    theme(
        # legend
        legend.title=element_blank(),
        # grid
        panel.grid=element_blank(),
        # strip
        strip.background=element_rect(fill="white"),
        strip.text=element_text(size=12, color="black"),
        # axes
        axis.title=element_text(size=12, color="black"),
        axis.text.y=element_text(size=9, color="black"),
        axis.text.x=element_text(size=9, color="black", angle=90, vjust=0.5, hjust=1)
    )
FIGS$crispr$spacers <-
    ggplot(data=TABS$crispr_aggr_sa, aes(x=crispr_tool, y=x, fill=asm_tool)) +
    geom_col(position="dodge") +
    scale_fill_manual(values=ASM_TOOL_COLORS, guide="legend") +
    labs(
        x="",
        y="Number of spacers"
    ) +
    crispr_theme
FIGS$crispr$arrays <-
    ggplot(data=TABS$crispr_aggr_aa, aes(x=crispr_tool, y=x, fill=asm_tool)) +
    geom_col(position="dodge") +
    scale_fill_manual(values=ASM_TOOL_COLORS, guide="legend") +
    labs(
        x="",
        y="Number of CRISPR arrays"
    ) +
    crispr_theme