#!/usr/bin/Rscript

# print(snakemake)

## LOG FILE
sink(file=file(snakemake@log$out, open="wt"), type="output")
sink(file=file(snakemake@log$err, open="wt"), type="message")

## NOTE
# XXX

## IMPORT
suppressMessages(library(testit)) # assertions
# suppressMessages(library(tools))
# suppressMessages(library(gtools))
# suppressMessages(library(rjson))
suppressMessages(library(rmarkdown)) #Rmarkdown
# suppressMessages(library(knitr))
# suppressMessages(library(DT))
# suppressMessages(require(grid))
suppressMessages(library(ggplot2)) # plotting
suppressMessages(library(reshape2)) # reshaping dataframes
suppressMessages(library(scales)) # plot scales
suppressMessages(library(pheatmap)) # heatmaps
suppressMessages(require(grid)) # for heatmaps
suppressMessages(library(viridis)) # color palette

# custom
source(snakemake@params$utils)

## XXX
report_file <- file.path(getwd(), snakemake@output$html)

## DATA
INFO <- list()
FIGS <- list()
TABS <- list()

source(snakemake@input$src)

## RENDER
print(sprintf("rendering report %s", report_file))
rmarkdown::render(snakemake@input$rmd, output_file=report_file)

## PDF

# Mappability
pdf(snakemake@output$pdf_mappability, width=7, height=5)
for(pp in FIGS$mappability){ print(pp) }
dev.off()

# Mash
pdf(snakemake@output$pdf_mash, width=7, height=7)
grid.newpage()
grid.draw(FIGS$mash)
dev.off()

# QUAST
pdf(snakemake@output$pdf_quast, width=7, height=10)
print(FIGS$quast)
dev.off()

# Plasflow
pdf(snakemake@output$pdf_plasflow, width=7, height=5)
for(pp in FIGS$plasflow){ print(pp) }
dev.off()

# Prodigal
pdf(snakemake@output$pdf_prodigal, width=7, height=5)
for(pp in FIGS$prodigal){ print(pp) }
dev.off()

# DIAMOND
pdf(snakemake@output$pdf_diamond, width=7, height=7)
for(pp in FIGS$diamond){ print(pp) }
dev.off()

# DIAMOND
pdf(snakemake@output$pdf_cdhit, width=7, height=7)
print(FIGS$cdhit)
dev.off()

# RGI
pdf(snakemake@output$pdf_rgi, width=7, height=5)
for(pp in FIGS$rgi){ print(pp) }
plot_rgi_overlap(TABS$rgi, "all", "ARO")
plot_rgi_overlap(TABS$rgi, "nudged", "ARO")
plot_rgi_overlap(TABS$rgi, "strict", "ARO")
dev.off()

# CRISPR
pdf(snakemake@output$pdf_crispr, width=7, height=5)
for(pp in FIGS$crispr){ print(pp) }
plot_crispr_overlap(TABS$crispr, "Flye")
plot_crispr_overlap(TABS$crispr, "MEGAHIT")
plot_crispr_overlap(TABS$crispr, "metaSPAdes")
plot_crispr_overlap(TABS$crispr, "metaSPAdes (H)")
plot_crispr_overlap(TABS$crispr, "OPERA-MS")
dev.off()
